//////////////////////////////////////////////////////////////////////////////
//
// Project:       Delayline Detector
//
// Module:        driver library (DLL)
// Date:          13.05.2008
// File:          DelayLineDefs.h
//
// Description:   definitions for DLL interface
//
//////////////////////////////////////////////////////////////////////////////

#ifndef _DelayLineDefs_h_
#define _DelayLineDefs_h_

#include "ErrorCodes.h"

typedef  unsigned long  DLD_ID;
typedef  unsigned long  DLD_DLL_MODE;
enum I2C_RT {I2C_RT_DS3902, I2C_RT_DS1805};


///////////////////////////////////////////////////////////
//
// image size definition
//
//    size:    number of pixels
//                1 .. (max. image size)
//                -1 if not available
//
struct dld_image_size {
   int   size_x;     // pixel in x
   int   size_y;     // pixel in y
   int   size_t;	 // pixel in t
};


///////////////////////////////////////////////////////////
//
// ROI definition
//
//    offset:  start index
//                0 .. (max. image size - 1)
//                -1 if no roi available
//    size:    number of pixels
//                1 .. (max. image size)
//      
struct dld_image_roi {
   int   offset_x;   // roi offset in x
   int   offset_y;   // roi offset in y
   int   offset_t;   // roi offset in t
   int   size_x;     // size in x
   int   size_y;     // size in y
   int   size_t;	 // size in t
};



///////////////////////////////////////////////////////////
//
// binning definition
//
//    binnings:   table of available binnings
//      
struct dld_image_binnings {
   int      table_size_x;  // size of binning table x
   int      table_size_y;  // size of binning table y
   int		table_size_t;  // size of binning table t
   int*     binnings_x;    // available binning in x
   int*     binnings_y;    // available binning in y
   int*     binnings_t;    // available binning in t
};


///////////////////////////////////////////////////////////
//
// binning settings
//
struct dld_image_binning {
   int      bin_x;
   int      bin_y;
   int      bin_t;
};




///////////////////////////////////////////////////////////
//
// DLD properties
//
//    bpp:        bits per pixel
//    size:       number of pixels
//    pix_size:   pixel size in mm = distance pixel center to pixel center 
//      
struct dld_properties {
   dld_image_size    image_max;  // max. image size
   int      bpp;                    // bits per pixel
   double   pix_size_x;          // pixel size in x in mm
   double   pix_size_y;          // pixel size in y in mm
   double	pix_size_t;			 // pixel size in t in ps

   int   channels_max;           // max. number of detector channels, -1 if not available

   long  exp_time_min;           // min. exposure time in ms
   long  exp_time_max;           // max. exposure time in ms
   long  exp_delay_min;          // min. delay before exposure time in ms
   long  exp_delay_max;          // max. delay before exposure time in ms
   long  exp_iterations_max;     // max. number of exposure iterations

   int	max_timehist_len;			// max. recommended number of time channels in time histogram
   int	max_time_channels;			// max. number of time channels from measurement device
   int  modulo_resolution_factor;	// factor to calculate modulo parameter with sufficient precision from time period in time channels 

   double  real_segment_size;			//size of real segments in mm, if 0 then real segments not available
   int	 channel_binning;		 //if >0 binning forming virtual segments from pix_size_x (1D-DLD and virt. segments of 3D-DLD)
};




///////////////////////////////////////////////////////////
//
// DLD device parameter
//
//    dnl:        nonlinear pixel correction (double)
//    highresmod: high resolution mode on/off (bool)
//    tdc_threshold: TDC threshold (long)
//      
struct dld_dev_params {
   double   dnl;						//was F1 TDC relevant, now used for TTL inputs of R-Mode GPX
   bool     highres_mode;				//was F1 TDC relevant, no new use
   int      tdc_threshold;				//was F1 TDC relevant, now R-Mode DNL tuning
   int		pair_num;					//to be vanished, is defined in DLL modes
   int      time_aperture;				//currently only for raw data mode
   bool		calib_intensity;			//is usefull here to be double, is also defined in DLL modes
   bool		calib_image;				//is usefull here to be double, is also defined in DLL modes
   int		start_threshold;			//was F1 TDC relevant, now R-Mode DNL tuning
   int		time_modulo;				//single period backfold for multi periodic stops (in device time pixels of total time sum)
   int		time_offset;				//length of unrespected time range before any output and histogramming takes place (in device time pixels)
   double   time_walk;					//tdc_resolution factor (between 0.1 and 1.9)
   int		pCorrections[17];			//FPGA parameters to shift stop time results (for 16 stop results + 1 channelmask, "no.17")
   int		pZeroTimeWindows[16];		//TimeWindow start for 16 predefined time slices (Bluhm application) may be removed later
   int		pWidthTimeWindows[16];		//TimeWindow width for 16 predefined time slices (Bluhm application) may be removed later
   bool		One_Gpx;					//switch between single and double GPX TDC
   bool		I_Mode;						//switch between I-Mode (82 ps, 8 stops per GPX) and R-Mode (27 ps, 2 stops per GPX)
   bool		Ext_Gpx_Start;				//TDCs started with external or internal time reference
   bool		DLD_1D;						//1D DLD mode, to be moved in DLL modes
   bool		DLD_3D;						//pseudo 3D (Bluhm) DLD mode, may be removed later
   int		Data_Format;				//TDC SUMDIF Transfer format: 0->9bit,9bit,14bit; 1->10bit,10bit,12bit; 2->11bit,11bit,10bit; 3->12bit,12bit,8bit
};




///////////////////////////////////////////////////////////
//
// DLD exposure 
//
struct dld_exposure {
   long  exposure;      // exposure time in ms
   long  delay;         // delay before exposure time in ms
   long  iterations;    // number of exposure iterations
   bool  newIntCal;		// renew intensity calibration file from next exposure
};


///////////////////////////////////////////////////////////
//
// DLD region of calibration
//
struct dld_cal_region {
	int x1;
	int x2;
	int y1;
	int y2;
};



///////////////////////////////////////////////////////////
//
// DLD count
//
typedef  unsigned int t_dld_pos;
typedef  unsigned int t_dld_time;

struct dld_count {
   t_dld_pos   x;					// x coordinate of hit
   t_dld_pos   y;					// y coordinate of hit
//   t_dld_time  timestamp;			// t coordinate of hit
   t_dld_time  tx;					// x timesum (only in raw mode)
   t_dld_time  ty;					// y timesum (only in raw mode)
};

struct dld_counts {
   long        len;					// hit capacity of dld_counts buffer
   long		   filled;				// number of dld_counts in buffer
   bool        overflow;			// overflow flag when hit capacity was to small
   bool        use_counts_files;    // save list mode files when hit capacity reached
   dld_count*  counts;				// single event data structure
};


struct dld_histo {
	long        sum_hist_len;
	double*     pSumHistX;			// only in raw data mode
	double*     pSumHistY;			// only in raw data mode

	long        sum_xy_hist_len;
	double*     pSumHistXplusY;		// plain time histogram
};


struct dld_rates {
	long        stop_rate_len;		// number of detector hardware specific rate channels 
	double*     stop_rates;			// buffer for detector hardware specific rate results

	long        pair_rate_len;		// number of count rate channels for recognized data pairs 
	double*     pair_rates;			// buffer for count rate results of recognized data pairs

	long        image_stack_rate;	// 2D/3D histogram countrate after all conditions
};

struct timeslice {
	int* image;						// data structure for time slices (x,y) forming (x, y, t) histograms, 
									// can be defined fragmented and freely from 8 bit to 32 bit dynamics
};

///////////////////////////////////////////////////////////
//
// DLD data channels 
//
#define  DLD_DATA_NO_DATA           0x000000			//no op.
#define  DLD_DATA_TIMESTAMP         0x000001			//channel for current PC time at start of exposure
#define  DLD_DATA_CHANNELS          0x000010			//data channel for multi channel detector counts
#define  DLD_DATA_IMAGE_INT         0x000100			//data channel for 2D/3D histograms
#define	 DLD_DATA_SLICED_IMG		0x001000			//corresponds to IMAGE_INT, but using fragmented 3D buffer
#define  DLD_DATA_COUNTS            0x002000			//data channel for list mode (single event data)
#define  DLD_DATA_RATES             0x004000			//data channel for detector specific count rates
#define  DLD_DATA_LMF               0x010000			//file based list mode
#define  DLD_DATA_HISTO             0x100000			//time histogram channel
#define  DLD_DATA_EXT_LMF           0x200000			//image buffer based list mode
#define	 DLD_DATA_TDC_STREAM		0x200000			//plain TDC results, like transferd via USB


///////////////////////////////////////////////////////////
//
// DLD dll modes
//
#define  DLD_DLL_USB                0x000000			//normal hardware reading mode 
#define  DLD_DLL_DUMMY              0x000001			//for simulation without hardware
#define  DLD_DLL_MULTI_HIT          0x000002			//multihit on (currently DLL_RAW_DATA (pairmode = 0) only)
#define  DLD_DLL_ROI_EXT	        0x000004			//ROI in hardware (older RBTs only, always on for newer RBTs)
#define  DLD_DLL_TESTMODE           0x000008			//zoom out = earlier binning 6x6 (only F1 TDC)
#define  DLD_DLL_CAL_XYT            0x000010			//calibrate x,y,t
#define  DLD_DLL_CAL_INTENSITY      0x000020			//calibrate intensity
#define  DLD_DLL_EXT_TRIGGER_IN     0x000040			//external hardware exposure trigger 
#define  DLD_DLL_FLIM_MODE          0x000080			//flim mode
#define  DLD_DLL_IMAGE_SUM_X        0x000100			//sumX check into image
#define  DLD_DLL_IMAGE_SUM_Y        0x000200			//sumY check into image
#define  DLD_DLL_MIRROR_X			0x000400			//mirror image on y axis (DLL_RAW_DATA (pairmode = 0) only)
#define  DLD_DLL_MIRROR_Y			0x000800			//mirror image on x axis (DLL_RAW_DATA (pairmode = 0) only)
#define  DLD_DLL_VIRTUAL_CHANNELS	0x001000			//virtual data channels from x line
#define  DLD_DLL_IMAGE_EXT			0x002000			//image histogram build on TDC memory (not implemented yet)
#define  DLD_DLL_RAW_DATA			0x004000			//build data channels from raw data transfer (replaces pairmode = 0)
#define  DLD_DLL_FOURFOLD_TDC		0x008000			//4fold TDC device mode (TDC dev_id 9,10,11,12)


///////////////////////////////////////////////////////////
//
// DLD data buffer
//
//    buf_id:        unique buffer id
//
//    prepared_for:  prepared data channels in buffer
//                      e.g. 0x00111 for timestamp and image (int)
//      
//    used_for:      used data channels in buffer accordingly to 
//                   channel selection
//                      e.g. 0x00101 for timestamp and image (int)
//
//    seq_id:        sequence number, set by device for sequence control     
//      
struct dld_buf {
   int         buf_id;							//caller responsible, no change by DLL 
   int         seq_id;							//caller responsible, no change by DLL 

   long        prepared_for_data_channels;		//caller responsible, no change by DLL but check
   long        used_setup_for_data_channels;	//DLL feedback
   long		   used_dll_mode;					//DLL feedback

   long        timestamp;						//data channel for system time stamp at exposure start
 
   int*        channels;						//data channel for multi-channel (sub-) system results
   long        channels_size;					//caller responsible, no change by DLL but check for byte size
   
   int*        image;							//data channel for 3D images (x, y, t histogram)
   long        image_size;						//caller responsible, no change by DLL but check for byte size
      
   dld_counts* counts;							//data channel for complete single event results

   dld_rates*  rates;							//data channel for hardware measured countrates

   char*	   listmodefile;					//filename for listmode of complete single event results

   dld_histo*  histogram;						//data channel for 1D time histograms

   unsigned int* tdc_stream_buffer;				//data channel for tdc data stream as transfered via USB
   long			 tdc_stream_buffer_size;		//size of stream buffer in multiples of 32 bits
   
   timeslice*  slices;							//data channel for time slices (x, y, t histogram)
   int*		   slices_size;						//array of three elements, for x, y, t. No change by DLL but check for sizes
   int		   slice_bpp;						//defines slice buffer dynamics in bit per pixel, no change by DLL
};


///////////////////////////////////////////////////////////
//
// DLD image transformation table
//
// must be changed
//
struct dld_img_transformation {
   int      trans_id;
   double*  scale;
};



///////////////////////////////////////////////////////////
//
// DLD device types
#define DLD_DEV_TYPE_DELAYLINE         1  // the only device type

// DLD debug level defs
const int    DLL_DEBUG_NONE                     = 0;
const int    DLL_DEBUG_STATISTICS               = 1;
const int    DLL_DEBUG_FULL                     = 2;
const int    DLL_DEBUG_MAX                      = 3;


// DLD DLL interface
const char   DLL_DLD_OPEN []                    = "DLD_OPEN";
const char   DLL_DLD_OPEN_EXT []                = "DLD_OPEN_EXT";
const char   DLL_DLD_CLOSE []                   = "DLD_CLOSE";
const char   DLL_DLD_RESET []                   = "DLD_RESET";
const char   DLL_DLD_GET_INFO_STRING []         = "DLD_GET_INFO_STRING";
const char   DLL_DLD_ERROR_STRING []            = "DLD_ERROR_STRING";
const char   DLL_DLD_GET_DEVICE_TYPE []         = "DLD_GET_DEVICE_TYPE";
const char   DLL_DLD_GET_DEVICE_INFO []         = "DLD_GET_DEVICE_INFO";
const char   DLL_DLD_GET_RUNNING []             = "DLD_GET_RUNNING";
const char   DLL_DLD_GET_IMAGE_SIZE []          = "DLD_GET_IMAGE_SIZE";
const char   DLL_DLD_SET_IMAGE_SIZE []          = "DLD_SET_IMAGE_SIZE";
const char   DLL_DLD_GET_IMG_ROI []             = "DLD_GET_IMG_ROI";
const char   DLL_DLD_SET_IMG_ROI []             = "DLD_SET_IMG_ROI";
const char   DLL_DLD_GET_IMG_BINNING_INFO_MAX []= "DLD_GET_IMG_BINNING_INFO_MAX";
const char   DLL_DLD_GET_IMG_BINNING_INFO []    = "DLD_GET_IMG_BINNING_INFO";
const char   DLL_DLD_GET_IMG_BINNING []         = "DLD_GET_IMG_BINNING";
const char   DLL_DLD_SET_IMG_BINNING []         = "DLD_SET_IMG_BINNING";
const char   DLL_DLD_GET_EXPOSURE_TIME []       = "DLD_GET_EXPOSURE_TIME";
const char   DLL_DLD_SET_EXPOSURE_TIME []       = "DLD_SET_EXPOSURE_TIME";
const char   DLL_DLD_GET_DATA_INFOS []          = "DLD_GET_DATA_INFOS";
const char   DLL_DLD_GET_DATA_SELECTION []      = "DLD_GET_DATA_SELECTION";
const char   DLL_DLD_SET_DATA_SELECTION []      = "DLD_SET_DATA_SELECTION";
const char   DLL_DLD_START []                   = "DLD_START";
const char   DLL_DLD_STOP []                    = "DLD_STOP";
const char   DLL_DLD_ABORT []                   = "DLD_ABORT";
const char   DLL_DLD_SUSPEND []                 = "DLD_SUSPEND";
const char   DLL_DLD_RESUME []                  = "DLD_RESUME";
const char   DLL_DLD_GET_STATUS []              = "DLD_GET_STATUS";
const char   DLL_DLD_ADD_BUFFER []              = "DLD_ADD_BUFFER";
const char   DLL_DLD_REMOVE_BUFFER []           = "DLD_REMOVE_BUFFER";
const char   DLL_DLD_GET_BUFFER_STATUS []       = "DLD_GET_BUFFER_STATUS";
const char   DLL_DLD_GET_MAX_BUFFERS []         = "DLD_GET_MAX_BUFFERS";
const char   DLL_DLD_GET_EMPTY_BUFFER_NUM []    = "DLD_GET_EMPTY_BUFFER_NUM";
const char   DLL_DLD_GET_FILLED_BUFFER_NUM []   = "DLD_GET_FILLED_BUFFER_NUM";
const char   DLL_DLD_GET_NEXT_FILLED_BUFFER []  = "DLD_GET_NEXT_FILLED_BUFFER";
const char   DLL_DLD_RESET_BUFFER_QUEUE []      = "DLD_RESET_BUFFER_QUEUE";
const char   DLL_DLD_GET_IMG_TRANSFORMATION []  = "DLD_GET_IMG_TRANSFORMATION";
const char   DLL_DLD_SET_IMG_TRANSFORMATION []  = "DLD_SET_IMG_TRANSFORMATION";
const char   DLL_DLD_GET_DEBUG_LEVEL []         = "DLD_GET_DEBUG_LEVEL";
const char   DLL_DLD_SET_DEBUG_LEVEL []         = "DLD_SET_DEBUG_LEVEL";
const char   DLL_DLD_GET_IMG_THRESHOLD []       = "DLD_GET_IMG_THRESHOLD";
const char   DLL_DLD_SET_IMG_THRESHOLD []       = "DLD_SET_IMG_THRESHOLD";
const char   DLL_DLD_GET_CHANNEL_THRESHOLD []   = "DLD_GET_CHANNEL_THRESHOLD";
const char   DLL_DLD_SET_CHANNEL_THRESHOLD []   = "DLD_SET_CHANNEL_THRESHOLD";
const char   DLL_DLD_GET_DEV_PARAMS []          = "DLD_GET_DEV_PARAMS";
const char   DLL_DLD_SET_DEV_PARAMS []          = "DLD_SET_DEV_PARAMS";
const char   DLL_DLD_READY_CNT[]				= "DLD_READY_CNT";
const char   DLL_DLD_SET_CNT[]					= "DLD_SET_CNT";
const char	 DLL_DLD_GET_CNT[]					= "DLD_GET_CNT";
const char   DLL_DLD_SET_CAL[]					= "DLD_SET_CAL";
const char   DLL_DLD_SET_I2C_RES[]				= "DLD_SET_I2C_RES";
const char   DLL_DLD_SET_QDLD_SHIFTS[]			= "DLD_SET_QDLD_SHIFTS";

#endif   // _DelayLineDefs_h_
